from torch import nn
from torch.nn import functional as F
from resnet import ResNet, Bottleneck


class FCNHead(nn.Sequential):
    def __init__(self, in_channels: int, channels: int) -> None:
        inter_channels = in_channels // 4
        layers = [
            nn.Conv2d(in_channels, inter_channels, 3, padding=1, bias=False),
            nn.BatchNorm2d(inter_channels),
            nn.ReLU(),
            nn.Dropout(0.1),
            nn.Conv2d(inter_channels, channels, 1),
        ]

        super().__init__(*layers)


class FCNSegmentation(nn.Module):

    def __init__(self, num_classes) -> None:
        super().__init__()

        self.backbone = ResNet(Bottleneck, [3, 4, 6, 3], replace_stride_with_dilation=[False, True, True], )
        self.classifier = FCNHead(2048, num_classes)

    def forward(self, x):
        input_shape = x.shape[-2:]
        x = self.backbone(x)

        x = self.classifier(x)
        x = F.interpolate(x, size=input_shape, mode="bilinear", align_corners=False)

        return x